#include <windows.h>
#include <stdio.h>

#include "snoop.h"
#include "globals.h"
#include "registry.h"

/************************************************************************
* 
*  functions about registry
*
************************************************************************/

char snoop_path[256]="Software\\snoop";

VOID SaveAutoSaveRegistry()
{
	// write to registry
	CRegistry reg;

	reg.CreateKey(HKEY_CURRENT_USER, snoop_path);

	// write registry
	int nAutoSave;
	if (gbSaveConfigExit) nAutoSave=1;
	else nAutoSave=0;
	reg.Write("AutoSave", nAutoSave);
}


VOID SaveRegistry(HWND hwnd, int n)
{
	if (n<=0 || n>NUM_CONFIG) return;

	char conf_path[256];
	sprintf(conf_path, "%s\\conf%02d", snoop_path, n);
	
	// write to registry
	CRegistry reg;

	reg.CreateKey(HKEY_CURRENT_USER, conf_path);

	// write registry
	int bTop;
	if (gbOnTop) bTop=1;
	else bTop=0;
	if (n==NUM_CONFIG) bTop=0;
	reg.Write("Top", bTop);

	int bGrid;
	if (gbGrid) bGrid=1;
	else bGrid=0;
	if (n==NUM_CONFIG) bGrid=0;
	reg.Write("Grid", bGrid);

	int bRuler;
	if (gbRuler) bRuler=1;
	else bRuler=0;
	if (n==NUM_CONFIG) bRuler=0;
	reg.Write("Ruler", bRuler);

	int bHighlight;
	if (gbRGBGraphHighlight) bHighlight=1;
	else bHighlight=0;
	if (n==NUM_CONFIG) bHighlight=1;
	reg.Write("Highlight", bHighlight);

	int bCenterMark;
	if (gbCenterHighlight) bCenterMark=1;
	else bCenterMark=0;
	if (n==NUM_CONFIG) bCenterMark=1;
	reg.Write("CenterMark", bCenterMark);

	int bInversey;
	if (gbInverseY) bInversey=1;
	else bInversey=0;
	if (n==NUM_CONFIG) bInversey=0;
	reg.Write("InverseY", bInversey);

	int bHex;
	if (gbDisplayHex) bHex=1;
	else bHex=0;
	if (n==NUM_CONFIG) bHex=0;
	reg.Write("Hex", bHex);

	int bBound;
	if (gbBound) bBound=1;
	else bBound=0;
	if (n==NUM_CONFIG) bBound=0;
	reg.Write("Bound", bBound);

	int nShift;
	nShift = gnFastDelta;
	if (n==NUM_CONFIG) nShift=FASTDELTA;
	reg.Write("Shift", nShift);

	int nX, nY;
	// get current X, Y
	RECT rect;
	GetWindowRect(hwnd,&rect);
	nX = rect.left;
	nY = rect.top;
	reg.Write("X", nX);
	reg.Write("Y", nY);

	int nW, nH;
	// get current W, H
	if (gnClientWidth>0 && gnClientHeight>0) {
		nW = rect.right-rect.left;
		nH = rect.bottom-rect.top-54;
	} else {
		nW = 20 * 8;
		nH = 20 * 8+SB_HEIGHT;
	}
	if (n==NUM_CONFIG) {
		nW=-1;
		nH=-1;
	}
	reg.Write("W", nW);
	reg.Write("H", nH);

	if (n==NUM_CONFIG) {
		reg.Write("Zoom", 8);
	} else {
		reg.Write("Zoom", gnZoom);
	}

	if (n==NUM_CONFIG) {
		reg.Write("Mode", MENU_GRAPH_OFF);
	} else {
		reg.Write("Mode", gnRGBGraphMode);
	}

	if (n==NUM_CONFIG) {
		reg.Write("WorkingMode", MENU_MOUSE_MODE);
	} else {
		reg.Write("WorkingMode", gnMode);
	}

//	int nAutoSave;
//	if (gbSaveConfigExit) nAutoSave=1;
//	else nAutoSave=0;
//	reg.Write("AutoSave", nAutoSave);
	
	SaveAutoSaveRegistry();
}

VOID LoadRegistry(HWND hwnd, HMENU hmenu, int n)
{
	// read auto save registry
	CRegistry reg2;

	if (reg2.VerifyKey(HKEY_CURRENT_USER, snoop_path)!=0) {
		// read registry
		reg2.Open(HKEY_CURRENT_USER, snoop_path);

		int nAutoSave=-1;
		reg2.Read("AutoSave", nAutoSave);
		if (nAutoSave==0) gbSaveConfigExit=FALSE;
		else if (nAutoSave==1) gbSaveConfigExit=TRUE;
	}

	if (n<=0 || n>NUM_CONFIG) return;

	// load the other registry
	char conf_path[256];
	sprintf(conf_path, "%s\\conf%02d", snoop_path, n);

	CRegistry reg;
	char cmdline[1024]="";

	if (n==NUM_CONFIG) {
		if (reg.VerifyKey(HKEY_CURRENT_USER, conf_path)==0)
			SaveRegistry(hwnd, NUM_CONFIG);
	}

	if (reg.VerifyKey(HKEY_CURRENT_USER, conf_path)==0) {
		return;
		// no registry, create one?
		// SaveRegistry(hwnd);
	}

	// read registry
	reg.Open(HKEY_CURRENT_USER, conf_path);

	int bTop=-1;
	reg.Read("Top", bTop);
	if (bTop==0) {
		strcat(cmdline, "-notop ");
	} else if (bTop==1) {
		strcat(cmdline, "-top ");
	}

	int bGrid=-1;
	reg.Read("Grid", bGrid);
	if (bGrid==0) {
		strcat(cmdline, "-nogrid ");
	} else if (bGrid==1) {
		strcat(cmdline, "-grid ");
	}

	int bRuler=-1;
	reg.Read("Ruler", bRuler);
	if (bRuler==0) {
		strcat(cmdline, "-noruler ");
	} else if (bRuler==1) {
		strcat(cmdline, "-ruler ");
	}

	int bHighlight=-1;
	reg.Read("Highlight", bHighlight);
	if (bHighlight==0) {
		strcat(cmdline, "-nohighlight ");
	} else if (bHighlight==1) {
		strcat(cmdline, "-highlight ");
	}

	int bCenterMark=-1;
	reg.Read("CenterMark", bCenterMark);
	if (bCenterMark==0) {
		strcat(cmdline, "-nocentermark ");
	} else if (bCenterMark==1) {
		strcat(cmdline, "-centermark ");
	}

	int bInversey=-1;
	reg.Read("InverseY", bInversey);
	if (bInversey==1) {
		strcat(cmdline, "-inversey ");
	}

	int bHex=-1;
	reg.Read("Hex", bHex);
	if (bHex==1) {
		strcat(cmdline, "-hex ");
	}

	int bBound=-1;
	reg.Read("Bound", bBound);
	if (bBound==1) {
		strcat(cmdline, "-bound ");
	}

	int nShift=-1;
	reg.Read("Shift", nShift);
	if (nShift>=0) {
		char buf[32];
		sprintf(buf, "-shift %d ", nShift);
		strcat(cmdline, buf);
	}

	int nX=-1;
	reg.Read("X", nX);
	int nY=-1;
	reg.Read("Y", nY);
	if (nX>=0 && nY>=0) {
		char buf[32];
		sprintf(buf, "-coord %d %d ", nX, nY);
		strcat(cmdline, buf);
	}

	int nH=-1;
	reg.Read("H", nH);
	int nW=-1;
	reg.Read("W", nW);
	if (nW>=0 && nH>=0) {
		char buf[32];
		sprintf(buf, "-size %d %d ", nW, nH);
		strcat(cmdline, buf);
	}

	int nZoom=-1;
	reg.Read("Zoom", nZoom);
	if (nZoom>0) {
		gnZoom=nZoom;
		gnZoomIndex = findIndex(gnZoom);
		CalcZoomedSize();
	}

	int nMode=-1;
	reg.Read("Mode", nMode);
	if (nMode>=0) {
		char buf[32];
		switch (nMode) {
			case MENU_GRAPH_OFF:
				sprintf(buf, "-o ");
				break;
			case MENU_GRAPH_HOR_STEP:
				sprintf(buf, "-H ");
				break;
			case MENU_GRAPH_HOR_LINEAR:
				sprintf(buf, "-h ");
				break;
			case MENU_GRAPH_VER_STEP:
				sprintf(buf, "-V ");
				break;
			case MENU_GRAPH_VER_LINEAR:
				sprintf(buf, "-v ");
				break;
			case MENU_GRAPH_STATISTICS:
				SendMessage(hwnd, WM_COMMAND, MENU_GRAPH_STATISTICS, 0L);
				break;
		}
		if (nMode!=MENU_GRAPH_STATISTICS)
			strcat(cmdline, buf);
	}

	int nWorkingMode=-1;
	reg.Read("WorkingMode", nWorkingMode);
	if (nWorkingMode>=0) {
		char buf[32];
		switch (nWorkingMode) {
			case MENU_ARROW_MODE:
				sprintf(buf, "-a ");
				break;
			case MENU_MOUSE_MODE:
				sprintf(buf, "-m ");
				break;
		}
		strcat(cmdline, buf);
	}

//	int nAutoSave=-1;
//	reg.Read("AutoSave", nAutoSave);
//	if (nAutoSave==0) gbSaveConfigExit=FALSE;
//	else if (nAutoSave==1) gbSaveConfigExit=TRUE;

	ProcessCommandLine(cmdline, hwnd, hmenu);

}


VOID RemoveRegistry()
{
	CRegistry reg;

	char conf_path[256];

	for (int i=1; i<=NUM_CONFIG; i++) {
		sprintf(conf_path, "%s\\conf%02d", snoop_path, i);
		reg.DeleteValueKey(HKEY_CURRENT_USER, conf_path);
	}

	reg.DeleteValueKey(HKEY_CURRENT_USER, snoop_path);
}

